@@ -13,6 +13,7 @@ from prompt_toolkit import PromptSession
 from prompt_toolkit.history import FileHistory
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
 from prompt_toolkit import print_formatted_text
+from prompt_toolkit.formatted_text import FormattedText
 from prompt_toolkit.key_binding.bindings.named_commands import (
     register as prompt_register,
 )
@@ -22,7 +23,7 @@ from .style import STYLE
 from .config import config, load_config_files
 from .processors import UserInputCommand, UpdateBottomProcessor, PasswordProcessor
 from .bottom import BottomToolbar
-from .utils import timer, exit
+from .utils import timer, exit, convert_formatted_text_to_bytes
 from .completers import IRedisCompleter
 from .lexer import IRedisLexer
 from . import __version__
@@ -73,7 +74,14 @@ def print_help_msg(command):
         click.echo(command.get_help(ctx))
 
 
-def write_result(text):
+def is_too_tall(text, max_height):
+    if isinstance(text, FormattedText):
+        text = convert_formatted_text_to_bytes(text)
+    lines = len(text.split(b"\n"))
+    return lines > max_height
+
+
+def write_result(text, max_height=None):
     """
     When config.raw set to True, write text(must be bytes in that case)
     directly to stdout, same if text is bytes.
@@ -81,13 +89,35 @@ def write_result(text):
     :param text: is_raw: bytes or str, not raw: FormattedText
     :is_raw: bool
     """
-    logger.info(f"write{type(text)}: {text[:10]}")
-    if config.raw or isinstance(text, bytes):
-        if isinstance(text, str):
+    logger.info(f"Print result {type(text)}: {text}"[:200])
+
+    # only handle bytes or FormattedText, if it's str, convert to bytes
+    if isinstance(text, str):
+        if config.decode:
             text = text.encode(config.decode)
+        else:
+            text = text.encode()
+
+    # using pager if too tall
+    if max_height and config.enable_pager and is_too_tall(text, max_height):
+        if isinstance(text, FormattedText):
+            text = convert_formatted_text_to_bytes(text)
+            os.environ["LESS"] = "-SRX"
+        # click.echo_via_pager only accepts str
+        if config.decode:
+            text = text.decode()
+        else:
+            text = text.decode()
+        click.echo_via_pager(text)
+        return
+
+    if isinstance(text, bytes):
         sys.stdout.buffer.write(text)
         sys.stdout.write("\n")
     else:
+        # TODO
+        # FormattedText to Stringbuffer
+        # click.echo_via_pager if output is too long
         print_formatted_text(text, end="", style=STYLE)
         print_formatted_text()
 
@@ -174,7 +204,11 @@ def repl(client, session, start_time):
         try:
             answers = client.send_command(command, session.completer)
             for answer in answers:
-                write_result(answer)
+                write_result(
+                    answer,
+                    # -1 is because 127.0.0.1:6379> takes one line
+                    session.output.get_size().rows - session.reserve_space_for_menu - 1,
+                )
         # Error with previous command or exception
         except Exception as e:
             logger.exception(e)
@@ -264,6 +298,7 @@ Use Redis URL to indicate connection(Can set with env `IREDIS_URL`), Example:
     unix://[[username]:[password]]@/path/to/socket.sock?db=0
 """
 SHELL = """Allow to run shell commands, default to True."""
+PAGER_HELP = """Using pager when output is too tall for your window, default to True."""
 
 # command line entry here...
 @click.command()
@@ -294,6 +329,7 @@ SHELL = """Allow to run shell commands, default to True."""
 @click.option("--raw/--no-raw", default=None, is_flag=True, help=RAW_HELP)
 @click.option("--rainbow/--no-rainbow", default=None, is_flag=True, help=RAINBOW)
 @click.option("--shell/--no-shell", default=None, is_flag=True, help=SHELL)
+@click.option("--pager/--no-pager", default=None, is_flag=True, help=PAGER_HELP)
 @click.version_option()
 @click.argument("cmd", nargs=-1)
 def gather_args(
@@ -312,6 +348,7 @@ def gather_args(
     url,
     socket,
     shell,
+    pager,
 ):
     """
     IRedis: Interactive Redis
@@ -350,6 +387,8 @@ def gather_args(
         config.rainbow = rainbow
     if shell is not None:
         config.shell = shell
+    if pager is not None:
+        config.enable_pager = pager
 
     return ctx
 
@@ -441,6 +480,7 @@ def main():
             for answer in client.send_command(line, None):
                 write_result(answer)
         return
+
     # no interactive mode, directly run a command
     if ctx.params["cmd"]:
         answers = client.send_command(" ".join(ctx.params["cmd"]), None)
